﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Random;

namespace HeuristicLab.Encodings.ParameterConfigurationEncoding {
  /// <summary>
  /// An operator which crosses two double values. TODO
  /// </summary>
  [Item("NormalDoubleValueCrossover", "An operator which crosses two double values. TODO")]
  [StorableClass]
  public class NormalDoubleValueCrossover : SingleSuccessorOperator, IDoubleValueCrossover, IStochasticOperator {
    public ILookupParameter<IRandom> RandomParameter {
      get { return (LookupParameter<IRandom>)Parameters["Random"]; }
    }

    #region Constructors and Cloning
    public NormalDoubleValueCrossover() : base() { }

    [StorableConstructor]
    protected NormalDoubleValueCrossover(bool deserializing) : base(deserializing) { }
    protected NormalDoubleValueCrossover(NormalDoubleValueCrossover original, Cloner cloner) : base(original, cloner) { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new NormalDoubleValueCrossover(this, cloner);
    }
    #endregion

    public void Apply(IRandom random, DoubleValue value, DoubleValue other, DoubleValueRange range) {
      value.Value = ApplyStatic(random, value, other, range).Value;
    }

    public static DoubleValue ApplyStatic(IRandom random, DoubleValue better, DoubleValue worse, DoubleValueRange range) {
      NormalDistributedRandom N = new NormalDistributedRandom(random, better.Value, Math.Abs(better.Value - worse.Value) / 3);
      var offspring = new DoubleValue();
      do {
        offspring.Value = N.NextDouble();
        offspring.Value = range.ApplyStepSize(offspring.Value);
      } while (!range.IsInRange(offspring.Value));

      return offspring;
    }
  }
}
